#include "bsp.h"
#include "tty.h"
#include "led.h"

static char buf[16];
static unsigned char bufp, bufg;
static volatile int bufc;

/* nRTS is the only input */
#define nRTS p6.b5

#define nCTS p6.b0
#define nDSR p6.b1
#define nDCD p6.b2
#define nRI  p6.b4

#define STOP()
#define START()

void __attribute__((interrupt))
s1rx_handler ()
{
  buf[bufp&15] = u1rb;
  bufp ++;
  bufc ++;
  if (bufc > 12)
    STOP();
}

void
tty_init ()
{
  // 7 and 5 are inputs, rest outputs */
  pd6.b = 0x5f;

  pur1 = 0x20; /* pull up p6.4 through p6.7 */

  START();

  /* Round up */
  u1brg = (CLOCK_HZ /*+ BAUD * 8*/) / (BAUD * 16) - 1;

  u1mr.b = 5;	/* UART, 8N1 */
  u1c0.b = 0;	/* LSB first, CMOS, buffered, f1 */
  u1c1.b = 5;	/* enable tx, rx */
  u1sr = 3;
  pmr.u1pinsel = 1;
  pd6.b7 = 0;

  s1ric = 5;

  wait_ms (1);
}

void
tty_flush ()
{
  while (bufc)
    ;
}

void
tty_putc (char c)
{
  int i;
  if (c == '\n')
    tty_putc ('\r');
  while (!u1c1.ti)
    ;
#if 1
  for (i=0; i<500; i++)
    asm("nop");
#endif
  u1tb = c;
}

void
tty_puts (char *s)
{
  while (*s)
    {
      tty_putc (*s);
      s++;
    }
}

void
tty_write (void *data, int len)
{
  char *d = (char *)data;
  while (len--)
    tty_putc (*d++);
}

write (int fd, void *data, int len)
{
  tty_write (data, len);
  return len;
}

/* returns number of characters available */
int
tty_ready (void)
{
  return bufc;
}

/* returns character */
int
tty_readc (void)
{
  int rv;
#if 1
  while (bufc == 0)
    ;
  rv = buf[bufg&15];
  bufg++;
  asm("fclr I");
  bufc--;
  if (bufc < 4)
    START();
  asm("fset I");
#else
  while (u1c1.ri)
    ;
  rv = u1rb;
#endif
  return rv;
}

#if 0
/* returns number of characters read */
int
tty_read (void *data, int len)
{
}
#endif
